@@ -31,6 +31,7 @@ bwplot(xx , df.test)
 
 dotplot(zz ~ xx | aa, df.test)
 dotplot(aa ~ xx | zz, df.test)
+
 dotplot(zz , df.test)
 dotplot(xx , df.test)
 
@@ -56,5 +57,17 @@ x = sample(1:3, 100, replace=TRUE)
 histogram( ~ x, breaks=c(0,1.5,2.5,3.5), type='count')
 
 
+
+fubar <- function() {
+    k <- 2
+    kkk <- 1:10
+    names(kkk) <- 1:10
+    data = list(x=kkk)
+    dotplot(x^k + rnorm(10), data, allow = T)
+}
+
+fubar()
+
+
 demo("lattice")
 
